"use strict";
var ExtensionMethods_1 = require("../ExtensionMethods");
var Uri_1 = require("../Uri");
var ExchangeCredentials = (function () {
    function ExchangeCredentials(userName, password) {
        if (userName === void 0) { userName = null; }
        if (password === void 0) { password = null; }
        this.UserName = null;
        this.Password = null;
        if (arguments.length > 0 && (ExtensionMethods_1.StringHelper.IsNullOrEmpty(userName) || ExtensionMethods_1.StringHelper.IsNullOrEmpty(password))) {
            throw new Error("ExchangeCredentials.ctor, must provide username and password value.");
        }
        this.UserName = userName;
        this.Password = password;
    }
    ExchangeCredentials.prototype.AdjustUrl = function (url) { return new Uri_1.Uri(ExchangeCredentials.GetUriWithoutSuffix(url)); };
    ExchangeCredentials.prototype.EmitExtraSoapHeaderNamespaceAliases = function (writer /*System.Xml.XmlWriter*/) { };
    ExchangeCredentials.GetUriWithoutSuffix = function (url) {
        var absoluteUri = url.AbsoluteUri;
        //ref: can not use WSSecurityBasedCredentials.WsSecurityPathSuffix, creates circular reference.
        var index = absoluteUri.toUpperCase().indexOf(/*WSSecurityBasedCredentials*/ ExchangeCredentials.WsSecurityPathSuffix.toUpperCase()); //, StringComparison.OrdinalIgnoreCase);
        if (index != -1) {
            return absoluteUri.substring(0, index);
        }
        return absoluteUri;
    };
    //PreAuthenticate(): any{ throw new Error("ExchangeCredentials.ts - PreAuthenticate : Not implemented.");}
    ExchangeCredentials.prototype.PrepareWebRequest = function (request /*IEwsHttpWebRequest*/) {
        request.headers["Authorization"] = "Basic " + ExtensionMethods_1.base64Helper.btoa(this.UserName + ":" + this.Password);
    };
    ExchangeCredentials.prototype.SerializeExtraSoapHeaders = function (writer /*System.Xml.XmlWriter*/, webMethodName) { };
    //NeedSignature: boolean;
    ExchangeCredentials.WsSecurityPathSuffix = "/wssecurity";
    return ExchangeCredentials;
}());
exports.ExchangeCredentials = ExchangeCredentials;
